@@ -8,78 +8,55 @@ calling these methods!
 """
 import datetime
 
+from django.contrib.auth import get_user_model
 from django.contrib.sites.models import Site
 from django.core.exceptions import FieldError
 from django.core.exceptions import PermissionDenied
 from django.core.exceptions import ValidationError
+from django.db import transaction
 from django.template.defaultfilters import slugify
 from django.template.loader import get_template
 from django.utils import six
+from django.utils.translation import activate
 
-from cms.admin.forms import save_permissions
+from cms import constants
 from cms.app_base import CMSApp
 from cms.apphook_pool import apphook_pool
 from cms.constants import TEMPLATE_INHERITANCE_MAGIC
 from cms.models.pagemodel import Page
-from cms.models.permissionmodels import (PageUser, PagePermission,
-    GlobalPagePermission, ACCESS_PAGE_AND_DESCENDANTS)
+from cms.models.permissionmodels import (PageUser, PagePermission, GlobalPagePermission,
+                                         ACCESS_PAGE_AND_DESCENDANTS)
 from cms.models.placeholdermodel import Placeholder
 from cms.models.pluginmodel import CMSPlugin
 from cms.models.titlemodels import Title
 from cms.plugin_base import CMSPluginBase
 from cms.plugin_pool import plugin_pool
-from cms.utils import copy_plugins
-from cms.utils.compat.dj import get_user_model
+from cms.utils import copy_plugins, get_current_site
 from cms.utils.conf import get_cms_setting
 from cms.utils.i18n import get_language_list
-from cms.utils.permissions import _thread_locals
+from cms.utils.page import get_available_slug
+from cms.utils.permissions import _thread_locals, current_user
 from menus.menu_pool import menu_pool
 
 
-#===============================================================================
-# Constants 
-#===============================================================================
-
-VISIBILITY_ALL = None
-VISIBILITY_USERS = 1
-VISIBILITY_STAFF = 2
-
 #===============================================================================
 # Helpers/Internals
 #===============================================================================
 
 
-def _generate_valid_slug(source, parent, language):
-    """
-    Generate a valid slug for a page from source for the given language.
-    Parent is passed so we can make sure the slug is unique for this level in
-    the page tree.
-    """
-    if parent:
-        qs = Title.objects.filter(language=language, page__parent=parent)
-    else:
-        qs = Title.objects.filter(language=language, page__parent__isnull=True)
-    used = qs.values_list('slug', flat=True)
-    baseslug = slugify(source)
-    slug = baseslug
-    i = 1
-    while slug in used:
-        slug = '%s-%s' % (baseslug, i)
-        i += 1
-    return slug
-
-
 def _verify_apphook(apphook, namespace):
     """
     Verifies the apphook given is valid and returns the normalized form (name)
     """
     apphook_pool.discover_apps()
-    if hasattr(apphook, '__module__') and issubclass(apphook, CMSApp):
+    if isinstance(apphook, CMSApp):
         try:
-            assert apphook in apphook_pool.apps.values()
+            assert apphook.__class__ in [app.__class__ for app in apphook_pool.apps.values()]
         except AssertionError:
             print(apphook_pool.apps.values())
             raise
+        apphook_name = apphook.__class__.__name__
+    elif hasattr(apphook, '__module__') and issubclass(apphook, CMSApp):
         return apphook.__name__
     elif isinstance(apphook, six.string_types):
         try:
@@ -101,8 +78,7 @@ def _verify_plugin_type(plugin_type):
     (plugin_model, plugin_type)
     """
     if (hasattr(plugin_type, '__module__') and
-        issubclass(plugin_type, CMSPluginBase)):
-        plugin_pool.set_plugin_meta()
+            issubclass(plugin_type, CMSPluginBase)):
         plugin_model = plugin_type.model
         assert plugin_type in plugin_pool.plugins.values()
         plugin_type = plugin_type.__name__
@@ -117,31 +93,26 @@ def _verify_plugin_type(plugin_type):
         raise TypeError('plugin_type must be CMSPluginBase subclass or string')
     return plugin_model, plugin_type
 
+
 #===============================================================================
-# Public API 
+# Public API
 #===============================================================================
 
+@transaction.atomic
 def create_page(title, template, language, menu_title=None, slug=None,
                 apphook=None, apphook_namespace=None, redirect=None, meta_description=None,
                 created_by='python-api', parent=None,
                 publication_date=None, publication_end_date=None,
                 in_navigation=False, soft_root=False, reverse_id=None,
                 navigation_extenders=None, published=False, site=None,
-                login_required=False, limit_visibility_in_menu=VISIBILITY_ALL,
-                position="last-child", overwrite_url=None, xframe_options=Page.X_FRAME_OPTIONS_INHERIT):
+                login_required=False, limit_visibility_in_menu=constants.VISIBILITY_ALL,
+                position="last-child", overwrite_url=None,
+                xframe_options=Page.X_FRAME_OPTIONS_INHERIT, page_title=None):
     """
     Create a CMS Page and it's title for the given language
-    
+
     See docs/extending_cms/api_reference.rst for more info
     """
-    # ugly permissions hack
-    if created_by and isinstance(created_by, get_user_model()):
-        _thread_locals.user = created_by
-
-        created_by = getattr(created_by, get_user_model().USERNAME_FIELD)
-    else:
-        _thread_locals.user = None
-
     # validate template
     if not template == TEMPLATE_INHERITANCE_MAGIC:
         assert template in [tpl[0] for tpl in get_cms_setting('TEMPLATES')]
@@ -149,21 +120,17 @@ def create_page(title, template, language, menu_title=None, slug=None,
 
     # validate site
     if not site:
-        site = Site.objects.get_current()
+        site = get_current_site()
     else:
         assert isinstance(site, Site)
 
     # validate language:
     assert language in get_language_list(site), get_cms_setting('LANGUAGES').get(site.pk)
 
-    # set default slug:
-    if not slug:
-        slug = _generate_valid_slug(title, parent, language)
-
     # validate parent
     if parent:
         assert isinstance(parent, Page)
-        parent = Page.objects.get(pk=parent.pk)
+        assert parent.publisher_is_draft
 
     # validate publication date
     if publication_date:
@@ -179,25 +146,33 @@ def create_page(title, template, language, menu_title=None, slug=None,
         assert navigation_extenders in menus
 
     # validate menu visibility
-    accepted_limitations = (VISIBILITY_ALL, VISIBILITY_USERS, VISIBILITY_STAFF)
+    accepted_limitations = (constants.VISIBILITY_ALL, constants.VISIBILITY_USERS, constants.VISIBILITY_ANONYMOUS)
     assert limit_visibility_in_menu in accepted_limitations
 
     # validate position
     assert position in ('last-child', 'first-child', 'left', 'right')
+    target_node = parent.node if parent else None
+
     # validate and normalize apphook
     if apphook:
         application_urls = _verify_apphook(apphook, apphook_namespace)
     else:
         application_urls = None
 
+    # ugly permissions hack
+    if created_by and isinstance(created_by, get_user_model()):
+        _thread_locals.user = created_by
+        created_by = getattr(created_by, get_user_model().USERNAME_FIELD)
+    else:
+        _thread_locals.user = None
+
     if reverse_id:
-        if Page.objects.drafts().filter(reverse_id=reverse_id, site=site).count():
+        if Page.objects.drafts().filter(reverse_id=reverse_id, node__site=site).exists():
             raise FieldError('A page with the reverse_id="%s" already exist.' % reverse_id)
 
     page = Page(
         created_by=created_by,
         changed_by=created_by,
-        parent=parent,
         publication_date=publication_date,
         publication_end_date=publication_end_date,
         in_navigation=in_navigation,
@@ -207,17 +182,18 @@ def create_page(title, template, language, menu_title=None, slug=None,
         template=template,
         application_urls=application_urls,
         application_namespace=apphook_namespace,
-        site=site,
         login_required=login_required,
         limit_visibility_in_menu=limit_visibility_in_menu,
-        xframe_options=xframe_options,    
+        xframe_options=xframe_options,
     )
-    page.insert_at(parent, position)
+    page.set_tree_node(site=site, target=target_node, position=position)
     page.save()
+    page.rescan_placeholders()
 
     create_title(
         language=language,
         title=title,
+        page_title=page_title,
         menu_title=menu_title,
         slug=slug,
         redirect=redirect,
@@ -229,53 +205,66 @@ def create_page(title, template, language, menu_title=None, slug=None,
     if published:
         page.publish(language)
 
+    if parent and position in ('last-child', 'first-child'):
+        parent._clear_node_cache()
+
     del _thread_locals.user
-    return page.reload()
+    return page
 
 
+@transaction.atomic
 def create_title(language, title, page, menu_title=None, slug=None,
-                 redirect=None, meta_description=None,
-                 parent=None, overwrite_url=None):
+                 redirect=None, meta_description=None, parent=None,
+                 overwrite_url=None, page_title=None, path=None):
     """
     Create a title.
-    
+
     Parent is only used if slug=None.
-    
+
     See docs/extending_cms/api_reference.rst for more info
     """
     # validate page
     assert isinstance(page, Page)
 
     # validate language:
-    assert language in get_language_list(page.site_id)
+    assert language in get_language_list(page.node.site_id)
 
     # set default slug:
     if not slug:
-        slug = _generate_valid_slug(title, parent, language)
+        base = page.get_path_for_slug(slugify(title), language)
+        slug = get_available_slug(page.node.site, base, language)
+
+    if overwrite_url:
+        path = overwrite_url.strip('/')
+    elif path is None:
+        path = page.get_path_for_slug(slug, language)
 
     title = Title.objects.create(
         language=language,
         title=title,
         menu_title=menu_title,
+        page_title=page_title,
         slug=slug,
+        path=path,
         redirect=redirect,
         meta_description=meta_description,
-        page=page
+        page=page,
+        has_url_overwrite=bool(overwrite_url),
     )
 
-    if overwrite_url:
-        title.has_url_overwrite = True
-        title.path = overwrite_url
-        title.save()
+    page_languages = page.get_languages()
 
+    if language not in page_languages:
+        page.update_languages(page_languages + [language])
     return title
 
 
+@transaction.atomic
 def add_plugin(placeholder, plugin_type, language, position='last-child',
                target=None, **data):
     """
     Add a plugin to a placeholder
-    
+
     See docs/extending_cms/api_reference.rst for more info
     """
     # validate placeholder
@@ -285,29 +274,58 @@ def add_plugin(placeholder, plugin_type, language, position='last-child',
     plugin_model, plugin_type = _verify_plugin_type(plugin_type)
     if target:
         if position == 'last-child':
-            new_pos = CMSPlugin.objects.filter(language=language, parent=target, tree_id=target.tree_id).count()
+            if CMSPlugin.node_order_by:
+                position = 'sorted-child'
+            new_pos = CMSPlugin.objects.filter(parent=target).count()
+            parent_id = target.pk
         elif position == 'first-child':
             new_pos = 0
+            if CMSPlugin.node_order_by:
+                position = 'sorted-child'
+            parent_id = target.pk
         elif position == 'left':
             new_pos = target.position
+            if CMSPlugin.node_order_by:
+                position = 'sorted-sibling'
+            parent_id = target.parent_id
         elif position == 'right':
             new_pos = target.position + 1
+            if CMSPlugin.node_order_by:
+                position = 'sorted-sibling'
+            parent_id = target.parent_id
         else:
             raise Exception('position not supported: %s' % position)
-        for pl in CMSPlugin.objects.filter(language=language, parent=target.parent_id, tree_id=target.tree_id, position__gte=new_pos):
+        if position == 'last-child' or position == 'first-child':
+            qs = CMSPlugin.objects.filter(language=language, parent=target, position__gte=new_pos,
+                                          placeholder=placeholder)
+        else:
+            qs = CMSPlugin.objects.filter(language=language, parent=target.parent_id, position__gte=new_pos,
+                                          placeholder=placeholder)
+        for pl in qs:
             pl.position += 1
             pl.save()
     else:
-        new_pos = CMSPlugin.objects.filter(language=language, parent__isnull=True, placeholder=placeholder).count()
-
+        if position == 'last-child':
+            new_pos = CMSPlugin.objects.filter(language=language, parent__isnull=True, placeholder=placeholder).count()
+        else:
+            new_pos = 0
+            for pl in CMSPlugin.objects.filter(language=language, parent__isnull=True, position__gte=new_pos,
+                                               placeholder=placeholder):
+                pl.position += 1
+                pl.save()
+        parent_id = None
     plugin_base = CMSPlugin(
         plugin_type=plugin_type,
         placeholder=placeholder,
         position=new_pos,
-        language=language
+        language=language,
+        parent_id=parent_id,
     )
-    plugin_base.insert_at(target, position=position, save=False)
 
+    plugin_base = plugin_base.add_root(instance=plugin_base)
+
+    if target:
+        plugin_base = plugin_base.move(target, pos=position)
     plugin = plugin_model(**data)
     plugin_base.set_base_attr(plugin)
     plugin.save()
@@ -324,9 +342,10 @@ def create_page_user(created_by, user,
                      can_delete_pagepermission=True, grant_all=False):
     """
     Creates a page user.
-    
+
     See docs/extending_cms/api_reference.rst for more info
     """
+    from cms.admin.forms import save_permissions
     if grant_all:
         # just be lazy
         return create_page_user(created_by, user, True, True, True, True,
@@ -367,7 +386,7 @@ def assign_user_to_page(page, user, grant_on=ACCESS_PAGE_AND_DESCENDANTS,
                         grant_all=False, global_permission=False):
     """
     Assigns given user to page, and gives him requested permissions.
-    
+
     See docs/extending_cms/api_reference.rst for more info
     """
     grant_all = grant_all and not global_permission
@@ -389,7 +408,7 @@ def assign_user_to_page(page, user, grant_on=ACCESS_PAGE_AND_DESCENDANTS,
         page_permission = GlobalPagePermission(
             user=user, can_recover_page=can_recover_page, **data)
         page_permission.save()
-        page_permission.sites.add(Site.objects.get_current())
+        page_permission.sites.add(get_current_site())
     return page_permission
 
 
@@ -397,22 +416,50 @@ def publish_page(page, user, language):
     """
     Publish a page. This sets `page.published` to `True` and calls publish()
     which does the actual publishing.
-    
+
     See docs/extending_cms/api_reference.rst for more info
     """
     page = page.reload()
 
-    class FakeRequest(object):
-        def __init__(self, user):
-            self.user = user
-
-    request = FakeRequest(user)
-    if not page.has_publish_permission(request):
+    if not page.has_publish_permission(user):
         raise PermissionDenied()
-    page.publish(language)
+    # Set the current_user to have the page's changed_by
+    # attribute set correctly.
+    # 'user' is a user object, but current_user() just wants the username (a string).
+    with current_user(user.get_username()):
+        page.publish(language)
     return page.reload()
 
 
+def publish_pages(include_unpublished=False, language=None, site=None):
+    """
+    Create published public version of selected drafts.
+    """
+    qs = Page.objects.drafts()
+
+    if not include_unpublished:
+        qs = qs.filter(title_set__published=True).distinct()
+
+    if site:
+        qs = qs.filter(node__site=site)
+
+    output_language = None
+    for i, page in enumerate(qs):
+        add = True
+        titles = page.title_set
+        if not include_unpublished:
+            titles = titles.filter(published=True)
+        for lang in titles.values_list("language", flat=True):
+            if language is None or lang == language:
+                if not output_language:
+                    output_language = lang
+                if not page.publish(lang):
+                    add = False
+        # we may need to activate the first (main) language for proper page title rendering
+        activate(output_language)
+        yield (page, add)
+
+
 def get_page_draft(page):
     """
     Returns the draft version of a page, regardless if the passed in
@@ -427,7 +474,7 @@ def get_page_draft(page):
         if page.publisher_is_draft:
             return page
         else:
-            return page.publisher_draft
+            return page.publisher_public
     else:
         return None
 
@@ -454,13 +501,32 @@ def copy_plugins_to_language(page, source_language, target_language,
     :return int: number of copied plugins
     """
     copied = 0
-    placeholders = page.placeholders.all()
+    placeholders = page.get_placeholders()
     for placeholder in placeholders:
         # only_empty is True we check if the placeholder already has plugins and
         # we skip it if has some
-        if not only_empty or not placeholder.cmsplugin_set.filter(language=target_language).exists():
+        if not only_empty or not placeholder.get_plugins(language=target_language).exists():
             plugins = list(
-                placeholder.cmsplugin_set.filter(language=source_language).order_by('tree_id', 'level', 'position'))
+                placeholder.get_plugins(language=source_language).order_by('path'))
             copied_plugins = copy_plugins.copy_plugins_to(plugins, placeholder, target_language)
             copied += len(copied_plugins)
     return copied
+
+
+def can_change_page(request):
+    """
+    Check whether a user has the permission to change the page.
+
+    This will work across all permission-related setting, with a unified interface
+    to permission checking.
+    """
+    from cms.utils import page_permissions
+
+    user = request.user
+    current_page = request.current_page
+
+    if current_page:
+        return page_permissions.user_can_change_page(user, current_page)
+
+    site = Site.objects.get_current(request)
+    return page_permissions.user_can_change_all_pages(user, site)
