@@ -4,19 +4,20 @@ from importlib import import_module
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
-from django.core.urlresolvers import (RegexURLResolver, Resolver404, reverse,
-                                      RegexURLPattern)
 from django.db import OperationalError, ProgrammingError
 from django.utils import six
 from django.utils.translation import get_language, override
+from django.urls import Resolver404, reverse
 
 from cms.apphook_pool import apphook_pool
 from cms.models.pagemodel import Page
 from cms.utils import get_current_site
-from cms.utils.compat import DJANGO_1_8, DJANGO_1_9
+from cms.utils.compat import DJANGO_1_11
+from cms.utils.compat.dj import RegexPattern, URLPattern, URLResolver
 from cms.utils.i18n import get_language_list
 from cms.utils.moderator import use_draft
 
+
 APP_RESOLVERS = []
 
 
@@ -59,7 +60,7 @@ def applications_page_check(request, current_page=None, path=None):
     return None
 
 
-class AppRegexURLResolver(RegexURLResolver):
+class AppRegexURLResolver(URLResolver):
     def __init__(self, *args, **kwargs):
         self.page_id = None
         self.url_patterns_dict = {}
@@ -79,7 +80,8 @@ class AppRegexURLResolver(RegexURLResolver):
         assigned.
         """
         tried = []
-        match = self.regex.search(path)
+        pattern = getattr(self, 'pattern', self)
+        match = pattern.regex.search(path)
         if match:
             new_path = path[match.end():]
             for pattern in self.url_patterns:
@@ -100,6 +102,7 @@ class AppRegexURLResolver(RegexURLResolver):
                     else:
                         if sub_match:
                             return getattr(pattern, 'page_id', None)
+                        pattern = getattr(pattern, 'pattern', pattern)
                         tried.append(pattern.regex.pattern)
             raise Resolver404({'tried': tried, 'path': new_path})
 
@@ -111,35 +114,36 @@ def recurse_patterns(path, pattern_list, page_id, default_args=None,
     """
     newpatterns = []
     for pattern in pattern_list:
-        app_pat = pattern.regex.pattern
+        app_pat = getattr(pattern, 'pattern', pattern).regex.pattern
         # make sure we don't get patterns that start with more than one '^'!
         app_pat = app_pat.lstrip('^')
         path = path.lstrip('^')
         regex = r'^%s%s' % (path, app_pat) if not nested else r'^%s' % (app_pat)
-        if isinstance(pattern, RegexURLResolver):
+        if isinstance(pattern, URLResolver):
             # include default_args
             args = pattern.default_kwargs
             if default_args:
                 args.update(default_args)
-            if DJANGO_1_8:
-                # this is an 'include', recurse!
-                resolver = RegexURLResolver(regex, 'cms_appresolver',
-                                            pattern.default_kwargs, pattern.app_name, pattern.namespace)
-                # see lines 243 and 236 of urlresolvers.py to understand the next line
-                resolver._urlconf_module = recurse_patterns(regex, pattern.url_patterns, page_id, args, nested=True)
-            else:
-                # see lines 243 and 236 of urlresolvers.py to understand the next line
-                urlconf_module = recurse_patterns(regex, pattern.url_patterns, page_id, args, nested=True)
-                # this is an 'include', recurse!
-                resolver = RegexURLResolver(regex, urlconf_module,
-                                            pattern.default_kwargs, pattern.app_name, pattern.namespace)
+            # see lines 243 and 236 of urlresolvers.py to understand the next line
+            urlconf_module = recurse_patterns(regex, pattern.url_patterns, page_id, args, nested=True)
+            # this is an 'include', recurse!
+            regex_pattern = regex
+            if not DJANGO_1_11:
+                regex_pattern = RegexPattern(regex)
+            resolver = URLResolver(regex_pattern, urlconf_module,
+                                   pattern.default_kwargs, pattern.app_name,
+                                   pattern.namespace)
         else:
-            # Re-do the RegexURLPattern with the new regular expression
+            # Re-do the URLPattern with the new regular expression
             args = pattern.default_args
             if default_args:
                 args.update(default_args)
-            resolver = RegexURLPattern(regex, pattern.callback,
-                                       args, pattern.name)
+
+            regex_pattern = regex
+            if not DJANGO_1_11:
+                regex_pattern = RegexPattern(regex, name=pattern.name)
+            resolver = URLPattern(regex_pattern, pattern.callback, args,
+                                  pattern.name)
         resolver.page_id = page_id
         newpatterns.append(resolver)
     return newpatterns
@@ -147,16 +151,13 @@ def recurse_patterns(path, pattern_list, page_id, default_args=None,
 
 def _set_permissions(patterns, exclude_permissions):
     for pattern in patterns:
-        if isinstance(pattern, RegexURLResolver):
+        if isinstance(pattern, URLResolver):
             if pattern.namespace in exclude_permissions:
                 continue
             _set_permissions(pattern.url_patterns, exclude_permissions)
         else:
             from cms.utils.decorators import cms_perms
-            if DJANGO_1_9:
-                pattern._callback = cms_perms(pattern.callback)
-            else:
-                pattern.callback = cms_perms(pattern.callback)
+            pattern.callback = cms_perms(pattern.callback)
 
 
 def get_app_urls(urls):
@@ -259,8 +260,9 @@ def _get_app_patterns(site):
         for lang in hooked_applications[page_id].keys():
             (app_ns, inst_ns), current_patterns, app = hooked_applications[page_id][lang]  # nopyflakes
             if not resolver:
+                regex_pattern = RegexPattern(r'') if not DJANGO_1_11 else r''
                 resolver = AppRegexURLResolver(
-                    r'', 'app_resolver', app_name=app_ns, namespace=inst_ns)
+                    regex_pattern, 'app_resolver', app_name=app_ns, namespace=inst_ns)
                 resolver.page_id = page_id
             if app.permissions:
                 _set_permissions(current_patterns, app.exclude_permissions)
