@@ -1,12 +1,13 @@
 # -*- coding: utf-8 -*-
-import six
-from django.utils.translation import ugettext_lazy as _
 from django import forms
 from django.contrib.admin.widgets import RelatedFieldWidgetWrapper
 from django.forms.fields import EMPTY_VALUES
-from cms.models.pagemodel import Page
-from cms.forms.widgets import PageSelectWidget, PageSmartLinkWidget
+from django.utils.translation import ugettext_lazy as _
+
 from cms.forms.utils import get_site_choices, get_page_choices
+from cms.forms.validators import validate_url
+from cms.forms.widgets import PageSelectWidget, PageSmartLinkWidget
+from cms.models.pagemodel import Page
 
 
 class SuperLazyIterator(object):
@@ -59,7 +60,7 @@ class PageSelectFormField(forms.MultiValueField):
             return Page.objects.get(pk=page_id)
         return None
 
-    def _has_changed(self, initial, data):
+    def has_changed(self, initial, data):
         is_empty = data and (len(data) >= 2 and data[1] in [None, ''])
 
         if isinstance(self.widget, RelatedFieldWidgetWrapper):
@@ -70,19 +71,28 @@ class PageSelectFormField(forms.MultiValueField):
             # this will cause django to always return True because of the '1'
             # so we simply follow django's default behavior when initial is None and data is "empty"
             data = ['' for x in range(0, len(data))]
-        return super(PageSelectFormField, self)._has_changed(initial, data)
+        return super(PageSelectFormField, self).has_changed(initial, data)
+
+    def _has_changed(self, initial, data):
+        return self.has_changed(initial, data)
+
 
 class PageSmartLinkField(forms.CharField):
     widget = PageSmartLinkWidget
+    default_validators = [validate_url]
 
     def __init__(self, max_length=None, min_length=None, placeholder_text=None,
                  ajax_view=None, *args, **kwargs):
         self.placeholder_text = placeholder_text
         widget = self.widget(ajax_view=ajax_view)
-        super(PageSmartLinkField, self).__init__(max_length, min_length,
+        super(PageSmartLinkField, self).__init__(max_length=max_length, min_length=min_length,
                                                  widget=widget, *args, **kwargs)
 
     def widget_attrs(self, widget):
         attrs = super(PageSmartLinkField, self).widget_attrs(widget)
-        attrs.update({'placeholder_text': six.text_type(self.placeholder_text)})
+        attrs.update({'placeholder_text': self.placeholder_text})
         return attrs
+
+    def clean(self, value):
+        value = self.to_python(value).strip()
+        return super(PageSmartLinkField, self).clean(value)
