@@ -1,37 +1,54 @@
 # -*- coding: utf-8 -*-
-from datetime import timedelta
-from cms.constants import PUBLISHER_STATE_DIRTY
-from cms.utils.compat.dj import python_2_unicode_compatible
 from django.db import models
 from django.utils import timezone
+from django.utils.encoding import python_2_unicode_compatible
 from django.utils.translation import ugettext_lazy as _
+
+from cms.constants import PUBLISHER_STATE_DIRTY
 from cms.models.managers import TitleManager
 from cms.models.pagemodel import Page
-from cms.utils.helpers import reversion_register
 
 
 @python_2_unicode_compatible
 class Title(models.Model):
+    # These are the fields whose values are compared when saving
+    # a Title object to know if it has changed.
+    editable_fields = [
+        'title',
+        'slug',
+        'redirect',
+        'page_title',
+        'menu_title',
+        'meta_description',
+        'has_url_overwrite',
+    ]
+
     language = models.CharField(_("language"), max_length=15, db_index=True)
     title = models.CharField(_("title"), max_length=255)
     page_title = models.CharField(_("title"), max_length=255, blank=True, null=True,
                                   help_text=_("overwrite the title (html title tag)"))
     menu_title = models.CharField(_("title"), max_length=255, blank=True, null=True,
                                   help_text=_("overwrite the title in the menu"))
-    meta_description = models.TextField(_("description"), max_length=155, blank=True, null=True,
+    meta_description = models.TextField(_("description"), blank=True, null=True,
                                         help_text=_("The text displayed in search engines."))
     slug = models.SlugField(_("slug"), max_length=255, db_index=True, unique=False)
     path = models.CharField(_("Path"), max_length=255, db_index=True)
     has_url_overwrite = models.BooleanField(_("has url overwrite"), default=False, db_index=True, editable=False)
-    redirect = models.CharField(_("redirect"), max_length=255, blank=True, null=True)
-    page = models.ForeignKey(Page, verbose_name=_("page"), related_name="title_set")
+    redirect = models.CharField(_("redirect"), max_length=2048, blank=True, null=True)
+    page = models.ForeignKey(Page, on_delete=models.CASCADE, verbose_name=_("page"), related_name="title_set")
     creation_date = models.DateTimeField(_("creation date"), editable=False, default=timezone.now)
 
     # Publisher fields
     published = models.BooleanField(_("is published"), blank=True, default=False)
     publisher_is_draft = models.BooleanField(default=True, editable=False, db_index=True)
     # This is misnamed - the one-to-one relation is populated on both ends
-    publisher_public = models.OneToOneField('self', related_name='publisher_draft', null=True, editable=False)
+    publisher_public = models.OneToOneField(
+        'self',
+        on_delete=models.CASCADE,
+        related_name='publisher_draft',
+        null=True,
+        editable=False,
+    )
     publisher_state = models.SmallIntegerField(default=0, editable=False, db_index=True)
 
     objects = TitleManager()
@@ -43,18 +60,23 @@ class Title(models.Model):
     def __str__(self):
         return u"%s (%s, %s)" % (self.title, self.slug, self.language)
 
-    def update_path(self):
-        # Build path from parent page's path and slug
-        slug = u'%s' % self.slug
-        if not self.has_url_overwrite:
-            self.path = u'%s' % slug
-            if self.page.parent_id:
-                parent_page = self.page.parent_id
+    def __repr__(self):
+        display = '<{module}.{class_name} id={id} is_draft={is_draft} object at {location}>'.format(
+            module=self.__module__,
+            class_name=self.__class__.__name__,
+            id=self.pk,
+            is_draft=self.publisher_is_draft,
+            location=hex(id(self)),
+        )
+        return display
 
-                parent_title = Title.objects.get_title(parent_page, language=self.language, language_fallback=True)
-                if parent_title:
-                    self.path = u'%s/%s' % (parent_title.path, slug)
+    def get_path_for_base(self, base_path=''):
+        old_base, sep, slug = self.path.rpartition('/')
+        return '%s/%s' % (base_path, slug) if base_path else slug
 
+    @property
+    def has_path_override(self):
+        return self.has_url_overwrite or bool(self.redirect)
 
     @property
     def overwrite_url(self):
@@ -77,42 +99,56 @@ class Title(models.Model):
         """
         keep_state = getattr(self, '_publisher_keep_state', None)
 
-         # Published pages should always have a publication date
-        # if the page is published we set the publish date if not set yet.
-        if self.page.publication_date is None and self.published:
-            self.page.publication_date = timezone.now() - timedelta(seconds=5)
-
         if self.publisher_is_draft and not keep_state and self.is_new_dirty():
             self.publisher_state = PUBLISHER_STATE_DIRTY
+
         if keep_state:
             delattr(self, '_publisher_keep_state')
-        ret = super(Title, self).save_base(*args, **kwargs)
-        return ret
+        return super(Title, self).save_base(*args, **kwargs)
 
     def is_new_dirty(self):
-        if self.pk:
-            fields = [
-                'title', 'page_title', 'menu_title', 'meta_description', 'slug', 'has_url_overwrite', 'redirect'
-            ]
-            try:
-                old_title = Title.objects.get(pk=self.pk)
-            except Title.DoesNotExist:
+        if not self.pk:
+            return True
+
+        try:
+            old_title = Title.objects.get(pk=self.pk)
+        except Title.DoesNotExist:
+            return True
+
+        for field in self.editable_fields:
+            old_val = getattr(old_title, field)
+            new_val = getattr(self, field)
+            if not old_val == new_val:
                 return True
-            for field in fields:
-                old_val = getattr(old_title, field)
-                new_val = getattr(self, field)
-                if not old_val == new_val:
-                    return True
-            return False
-        return True
+
+        if old_title.path != self.path and self.has_url_overwrite:
+            # path is handled individually because its a special field.
+            # The path field is both an internal and user facing field,
+            # as such we can't mark the title as dirty on any change,
+            # instead we need to check if the url overwrite flag is set.
+            return True
+        return False
+
+    def _url_properties_changed(self):
+        assert self.publisher_is_draft
+        assert self.publisher_public_id
+
+        new_values = (
+            self.path,
+            self.slug,
+            self.published,
+        )
+        old_values = (
+            self.publisher_public.path,
+            self.publisher_public.slug,
+            self.publisher_public.published,
+        )
+        return old_values != new_values
 
 
 class EmptyTitle(object):
-
-    def __init__(self, language):
-        self.language = language
-
-    """Empty title object, can be returned from Page.get_title_obj() if required
+    """
+    Empty title object, can be returned from Page.get_title_obj() if required
     title object doesn't exists.
     """
     title = ""
@@ -126,9 +162,17 @@ class EmptyTitle(object):
     page_title = ""
     published = False
 
+    def __init__(self, language):
+        self.language = language
+
+    def __nonzero__(self):
+        # Python 2 compatibility
+        return False
+
+    def __bool__(self):
+        # Python 3 compatibility
+        return False
+
     @property
     def overwrite_url(self):
         return None
-
-
-reversion_register(Title)
