@@ -2,24 +2,25 @@
 from django.db import models
 from cms.publisher.query import PublisherQuerySet
 
+
 class PublisherManager(models.Manager):
     """Manager with some support handling publisher.
     """
-    def get_query_set(self):
+    def get_queryset(self):
         """Change standard model queryset to our own.
         """
         return PublisherQuerySet(self.model)
-    
+
     def drafts(self):
-        return self.filter(publisher_is_draft=True)
-    
+        return self.get_queryset().drafts()
+
     def public(self):
-        return self.filter(publisher_is_draft=False)
-    
+        return self.get_queryset().public()
+
     """
     def all(self):
         raise NotImplementedError, ("Calling all() on manager of publisher "
             "object is not allowed. Please use drafts() or public() method "
-            "instead. If this isn't accident use get_query_set().all() for " 
+            "instead. If this isn't accident use get_queryset().all() for "
             "all instances.")
     """
