@@ -5,6 +5,8 @@ from django.db.models import Q
 from django.utils import translation
 
 from cms.models import Title
+from cms.utils import get_current_site
+from cms.utils.i18n import get_public_languages
 
 
 def from_iterable(iterables):
@@ -44,11 +46,14 @@ class CMSSitemap(Sitemap):
         # If, for some reason, you require redirecting pages (Titles) to be
         # included, simply create a new class inheriting from this one, and
         # supply a new items() method which doesn't filter out the redirects.
-        #
+        site = get_current_site()
+        languages = get_public_languages(site_id=site.pk)
         all_titles = Title.objects.public().filter(
             Q(redirect='') | Q(redirect__isnull=True),
-            page__login_required=False
-        ).order_by('page__tree_id', 'page__lft')
+            language__in=languages,
+            page__login_required=False,
+            page__node__site=site,
+        ).order_by('page__node__path')
         return all_titles
 
     def lastmod(self, title):
