@@ -2,66 +2,27 @@
 import sys
 from contextlib import contextmanager
 from shutil import rmtree as _rmtree
-from tempfile import template, mkdtemp
+from tempfile import template, mkdtemp, _exists
+from cms.apphook_pool import apphook_pool
 
-from django.conf import settings
-from django.core.signals import request_started
-from django.db import reset_queries
-from django.template import context
+from django.contrib.auth import get_user_model
 from django.utils.six.moves import StringIO
 from django.utils.translation import get_language, activate
 
-from cms.utils.compat.dj import get_user_model
-
 
 class NULL:
     pass
 
 
-class SettingsOverride(object):
-    """
-    Overrides Django settings within a context and resets them to their inital
-    values on exit.
-    
-    Example:
-    
-        with SettingsOverride(DEBUG=True):
-            # do something
-    """
-    
-    def __init__(self, **overrides):
-        self.overrides = overrides
-        self.special_handlers = {
-            'TEMPLATE_CONTEXT_PROCESSORS': self.template_context_processors,
-        }
-        
-    def __enter__(self):
-        self.old = {}
-        for key, value in self.overrides.items():
-            self.old[key] = getattr(settings, key, NULL)
-            setattr(settings, key, value)
-        
-    def __exit__(self, type, value, traceback):
-        for key, value in self.old.items():
-            if value is not NULL:
-                setattr(settings, key, value)
-            else:
-                delattr(settings,key) # do not pollute the context!
-            self.special_handlers.get(key, lambda:None)()
-    
-    def template_context_processors(self):
-        context._standard_context_processors = None
-
-
 class StdOverride(object):
     def __init__(self, std='out', buffer=None):
         self.std = std
         self.buffer = buffer or StringIO()
-        
+
     def __enter__(self):
         setattr(sys, 'std%s' % self.std, self.buffer)
         return self.buffer
-        
+
     def __exit__(self, type, value, traceback):
         setattr(sys, 'std%s' % self.std, getattr(sys, '__std%s__' % self.std))
 
@@ -69,7 +30,7 @@ class StdoutOverride(StdOverride):
     """
     This overrides Python's the standard output and redirects it to a StringIO
     object, so that on can test the output of the program.
-    
+
     example:
     lines = None
     with StdoutOverride() as buffer:
@@ -83,11 +44,11 @@ class StdoutOverride(StdOverride):
 class LanguageOverride(object):
     def __init__(self, language):
         self.newlang = language
-        
+
     def __enter__(self):
         self.oldlang = get_language()
         activate(self.newlang)
-        
+
     def __exit__(self, type, value, traceback):
         activate(self.oldlang)
 
@@ -111,12 +72,8 @@ class TemporaryDirectory:
         return self.name
 
     def cleanup(self):
-        try:
-            from tempfile import _exists
-            if _exists(self.name):
-                _rmtree(self.name)
-        except ImportError:
-            pass
+        if _exists(self.name):
+            _rmtree(self.name)
 
     def __exit__(self, exc, value, tb):
         self.cleanup()
@@ -126,14 +83,14 @@ class UserLoginContext(object):
     def __init__(self, testcase, user):
         self.testcase = testcase
         self.user = user
-        
+
     def __enter__(self):
-        loginok = self.testcase.client.login(username=getattr(self.user, get_user_model().USERNAME_FIELD), 
+        loginok = self.testcase.client.login(username=getattr(self.user, get_user_model().USERNAME_FIELD),
                                              password=getattr(self.user, get_user_model().USERNAME_FIELD))
         self.old_user = getattr(self.testcase, 'user', None)
         self.testcase.user = self.user
         self.testcase.assertTrue(loginok)
-        
+
     def __exit__(self, exc, value, tb):
         self.testcase.user = self.old_user
         if not self.testcase.user:
@@ -144,20 +101,20 @@ class UserLoginContext(object):
 class ChangeModel(object):
     """
     Changes attributes on a model while within the context.
-    
+
     These changes *ARE* saved to the database for the context!
     """
     def __init__(self, instance, **overrides):
         self.instance = instance
         self.overrides = overrides
-        
+
     def __enter__(self):
         self.old = {}
         for key, value in self.overrides.items():
             self.old[key] = getattr(self.instance, key, NULL)
             setattr(self.instance, key, value)
         self.instance.save()
-        
+
     def __exit__(self, exc, value, tb):
         for key in self.overrides.keys():
             old_value = self.old[key]
@@ -167,36 +124,6 @@ class ChangeModel(object):
                 setattr(self.instance, key, old_value)
         self.instance.save()
 
-class _AssertNumQueriesContext(object):
-    def __init__(self, test_case, num, connection):
-        self.test_case = test_case
-        self.num = num
-        self.connection = connection
-
-    def __enter__(self):
-        self.old_debug = settings.DEBUG
-        settings.DEBUG = True
-        self.starting_queries = len(self.connection.queries)
-        request_started.disconnect(reset_queries)
-        return self
-
-    def __exit__(self, exc_type, exc_value, traceback):
-        settings.DEBUG = self.old_debug
-        request_started.connect(reset_queries)
-        if exc_type is not None:
-            return
-
-        final_queries = len(self.connection.queries)
-        executed = final_queries - self.starting_queries
-        
-        queries = '\n'.join([q['sql'] for q in self.connection.queries[self.starting_queries:]])
-
-        self.test_case.assertEqual(
-            executed, self.num, "%d queries executed, %d expected. Queries executed:\n%s" % (
-                executed, self.num, queries
-            )
-        )
-
 
 @contextmanager
 def disable_logger(logger):
@@ -204,3 +131,44 @@ def disable_logger(logger):
     logger.disabled = True
     yield
     logger.disabled = old
+
+
+@contextmanager
+def apphooks(*hooks):
+    _apphooks = apphook_pool.apphooks
+    _apps = apphook_pool.apps
+    _discovered = apphook_pool.discovered
+    apphook_pool.clear()
+    for hook in hooks:
+        apphook_pool.register(hook)
+    try:
+        yield
+    finally:
+        apphook_pool.apphooks = _apphooks
+        apphook_pool.apps = _apps
+        apphook_pool.discovered = _discovered
+
+
+@contextmanager
+def signal_tester(*signals):
+    env = SignalTester()
+
+    for signal in signals:
+        signal.connect(env)
+
+    try:
+        yield env
+    finally:
+        for signal in signals:
+            signal.disconnect(env)
+
+
+class SignalTester(object):
+
+    def __init__(self):
+        self.call_count = 0
+        self.calls = []
+
+    def __call__(self, *args, **kwargs):
+        self.call_count += 1
+        self.calls.append((args, kwargs))
