@@ -4,9 +4,9 @@ from collections import defaultdict
 
 from django.template.loader import render_to_string
 from django.utils import six
+from django.utils.encoding import force_text
 from django.utils.functional import Promise
 
-from cms.utils.compat.dj import force_unicode
 from cms.constants import RIGHT, LEFT, REFRESH_PAGE, URL_CHANGE
 
 
@@ -37,7 +37,7 @@ class ToolbarAPIMixin(six.with_metaclass(ABCMeta)):
     URL_CHANGE = URL_CHANGE
     LEFT = LEFT
     RIGHT = RIGHT
-    
+
     def __init__(self):
         self.items = []
         self.menus = {}
@@ -159,7 +159,7 @@ class ToolbarAPIMixin(six.with_metaclass(ABCMeta)):
 
     def add_ajax_item(self, name, action, active=False, disabled=False,
                       extra_classes=None, data=None, question=None,
-                      side=LEFT, position=None, on_success=None):
+                      side=LEFT, position=None, on_success=None, method='POST'):
         item = AjaxItem(name, action, self.csrf_token,
                         active=active,
                         disabled=disabled,
@@ -168,12 +168,14 @@ class ToolbarAPIMixin(six.with_metaclass(ABCMeta)):
                         question=question,
                         side=side,
                         on_success=on_success,
+                        method=method,
         )
         self.add_item(item, position=position)
         return item
 
 
 class BaseItem(six.with_metaclass(ABCMeta)):
+    toolbar = None
     template = None
 
     def __init__(self, side=LEFT):
@@ -184,6 +186,10 @@ class BaseItem(six.with_metaclass(ABCMeta)):
         return self.side is RIGHT
 
     def render(self):
+        if self.toolbar:
+            template = self.toolbar.templates.get_cached_template(self.template)
+            return template.render(self.get_context())
+        # Backwards compatibility
         return render_to_string(self.template, self.get_context())
 
     def get_context(self):
@@ -191,6 +197,7 @@ class BaseItem(six.with_metaclass(ABCMeta)):
 
 
 class TemplateItem(BaseItem):
+
     def __init__(self, template, extra_context=None, side=LEFT):
         super(TemplateItem, self).__init__(side)
         self.template = template
@@ -207,14 +214,15 @@ class SubMenu(ToolbarAPIMixin, BaseItem):
     sub_level = True
     active = False
 
-    def __init__(self, name, csrf_token, side=LEFT):
+    def __init__(self, name, csrf_token, disabled=False, side=LEFT):
         ToolbarAPIMixin.__init__(self)
         BaseItem.__init__(self, side)
         self.name = name
+        self.disabled = disabled
         self.csrf_token = csrf_token
 
     def __repr__(self):
-        return '<Menu:%s>' % force_unicode(self.name)
+        return '<Menu:%s>' % force_text(self.name)
 
     def add_break(self, identifier=None, position=None):
         item = Break(identifier)
@@ -222,11 +230,17 @@ class SubMenu(ToolbarAPIMixin, BaseItem):
         return item
 
     def get_items(self):
-        return self.items
+        items = self.items
+        for item in items:
+            item.toolbar = self.toolbar
+            if hasattr(item, 'disabled'):
+                item.disabled = self.disabled or item.disabled
+        return items
 
     def get_context(self):
         return {
             'active': self.active,
+            'disabled': self.disabled,
             'items': self.get_items(),
             'title': self.name,
             'sub_level': self.sub_level
@@ -236,10 +250,10 @@ class SubMenu(ToolbarAPIMixin, BaseItem):
 class Menu(SubMenu):
     sub_level = False
 
-    def get_or_create_menu(self, key, verbose_name, side=LEFT, position=None):
+    def get_or_create_menu(self, key, verbose_name, disabled=False, side=LEFT, position=None):
         if key in self.menus:
             return self.menus[key]
-        menu = SubMenu(verbose_name, self.csrf_token, side=side)
+        menu = SubMenu(verbose_name, self.csrf_token, disabled=disabled, side=side)
         self.menus[key] = menu
         self.add_item(menu, position=position)
         return menu
@@ -257,7 +271,7 @@ class LinkItem(BaseItem):
         self.extra_classes = extra_classes or []
 
     def __repr__(self):
-        return '<LinkItem:%s>' % force_unicode(self.name)
+        return '<LinkItem:%s>' % force_text(self.name)
 
     def get_context(self):
         return {
@@ -269,12 +283,13 @@ class LinkItem(BaseItem):
         }
 
 
-class SideframeItem(BaseItem):
-    template = "cms/toolbar/items/item_sideframe.html"
+class FrameItem(BaseItem):
+    # Be sure to define the correct template
 
-    def __init__(self, name, url, active=False, disabled=False, extra_classes=None, on_close=None, side=LEFT):
-        super(SideframeItem, self).__init__(side)
-        self.name = "%s ..." % force_unicode(name)
+    def __init__(self, name, url, active=False, disabled=False,
+                 extra_classes=None, on_close=None, side=LEFT):
+        super(FrameItem, self).__init__(side)
+        self.name = "%s..." % force_text(name)
         self.url = url
         self.active = active
         self.disabled = disabled
@@ -282,7 +297,8 @@ class SideframeItem(BaseItem):
         self.on_close = on_close
 
     def __repr__(self):
-        return '<SideframeItem:%s>' % force_unicode(self.name)
+        # Should be overridden
+        return '<FrameItem:%s>' % force_text(self.name)
 
     def get_context(self):
         return {
@@ -295,11 +311,18 @@ class SideframeItem(BaseItem):
         }
 
 
-class ModalItem(SideframeItem):
+class SideframeItem(FrameItem):
+    template = "cms/toolbar/items/item_sideframe.html"
+
+    def __repr__(self):
+        return '<SideframeItem:%s>' % force_text(self.name)
+
+
+class ModalItem(FrameItem):
     template = "cms/toolbar/items/item_modal.html"
 
     def __repr__(self):
-        return '<ModalItem:%s>' % force_unicode(self.name)
+        return '<ModalItem:%s>' % force_text(self.name)
 
 
 class AjaxItem(BaseItem):
@@ -307,7 +330,7 @@ class AjaxItem(BaseItem):
 
     def __init__(self, name, action, csrf_token, data=None, active=False,
                  disabled=False, extra_classes=None,
-                 question=None, side=LEFT, on_success=None):
+                 question=None, side=LEFT, on_success=None, method='POST'):
         super(AjaxItem, self).__init__(side)
         self.name = name
         self.action = action
@@ -318,29 +341,30 @@ class AjaxItem(BaseItem):
         self.extra_classes = extra_classes or []
         self.question = question
         self.on_success = on_success
+        self.method = method
 
     def __repr__(self):
-        return '<AjaxItem:%s>' % force_unicode(self.name)
+        return '<AjaxItem:%s>' % force_text(self.name)
 
     def get_context(self):
-        data = {}
-        data.update(self.data)
-        data['csrfmiddlewaretoken'] = self.csrf_token
-        data = json.dumps(data)
+        data = self.data.copy()
+
+        if self.method not in ('GET', 'HEAD', 'OPTIONS', 'TRACE'):
+            data['csrfmiddlewaretoken'] = self.csrf_token
+
         return {
             'action': self.action,
             'name': self.name,
             'active': self.active,
             'disabled': self.disabled,
             'extra_classes': self.extra_classes,
-            'data': data,
+            'data': json.dumps(data),
             'question': self.question,
-            'on_success': self.on_success
+            'on_success': self.on_success,
+            'method': self.method,
         }
 
 
-
-
 class Break(BaseItem):
     template = "cms/toolbar/items/break.html"
 
@@ -349,9 +373,14 @@ class Break(BaseItem):
 
 
 class BaseButton(six.with_metaclass(ABCMeta)):
+    toolbar = None
     template = None
 
     def render(self):
+        if self.toolbar:
+            template = self.toolbar.templates.get_cached_template(self.template)
+            return template.render(self.get_context())
+        # Backwards compatibility
         return render_to_string(self.template, self.get_context())
 
     def get_context(self):
@@ -370,7 +399,7 @@ class Button(BaseButton):
         self.extra_classes = extra_classes or []
 
     def __repr__(self):
-        return '<Button:%s>' % force_unicode(self.name)
+        return '<Button:%s>' % force_text(self.name)
 
     def get_context(self):
         return {
@@ -394,7 +423,7 @@ class ModalButton(Button):
         self.on_close = on_close
 
     def __repr__(self):
-        return '<ModalButton:%s>' % force_unicode(self.name)
+        return '<ModalButton:%s>' % force_text(self.name)
 
     def get_context(self):
         return {
@@ -411,7 +440,7 @@ class SideframeButton(ModalButton):
     template = "cms/toolbar/items/button_sideframe.html"
 
     def __repr__(self):
-        return '<SideframeButton:%s>' % force_unicode(self.name)
+        return '<SideframeButton:%s>' % force_text(self.name)
 
 
 class ButtonList(BaseItem):
@@ -461,8 +490,68 @@ class ButtonList(BaseItem):
         self.buttons.append(item)
         return item
 
+    def get_buttons(self):
+        for button in self.buttons:
+            button.toolbar = self.toolbar
+            yield button
+
     def get_context(self):
-        return {
-            'buttons': self.buttons,
+        context = {
+            'buttons': list(self.get_buttons()),
             'extra_classes': self.extra_classes
         }
+
+        if self.toolbar:
+            context['cms_structure_on'] = self.toolbar.structure_mode_url_on
+        return context
+
+
+class Dropdown(ButtonList):
+
+    template = "cms/toolbar/items/dropdown.html"
+
+    def __init__(self, *args, **kwargs):
+        super(Dropdown, self).__init__(*args, **kwargs)
+        self.primary_button = None
+
+    def __repr__(self):
+        return '<Dropdown:%s>' % force_text(self.name)
+
+    def add_primary_button(self, button):
+        self.primary_button = button
+
+    def get_buttons(self):
+        for button in self.buttons:
+            button.toolbar = self.toolbar
+            button.is_in_dropdown = True
+            yield button
+
+    def get_context(self):
+        return {
+            'primary_button': self.primary_button,
+            'buttons': list(self.get_buttons()),
+            'extra_classes': self.extra_classes,
+        }
+
+
+class DropdownToggleButton(BaseButton):
+    template = "cms/toolbar/items/dropdown_button.html"
+    has_no_action = True
+
+    def __init__(self, name, active=False, disabled=False,
+                 extra_classes=None):
+        self.name = name
+        self.active = active
+        self.disabled = disabled
+        self.extra_classes = extra_classes or []
+
+    def __repr__(self):
+        return '<DropdownToggleButton:%s>' % force_text(self.name)
+
+    def get_context(self):
+        return {
+            'name': self.name,
+            'active': self.active,
+            'disabled': self.disabled,
+            'extra_classes': self.extra_classes,
+        }
