@@ -1,57 +1,57 @@
 # -*- coding: utf-8 -*-
-from cms.utils.conf import get_cms_setting
-from django.conf import settings
-from django.core.urlresolvers import reverse
-from django.utils.http import urlencode
-from cms.utils.compat.urls import urlparse
-from cms.utils.compat.dj import force_unicode
 import re
 
+from django.conf import settings
+from django.urls import reverse
+from django.utils.encoding import force_text
+from django.utils.http import urlencode
+from django.utils.six.moves.urllib.parse import urlparse
+
+import cms
+
+from cms.utils.conf import get_cms_setting
+
 # checks validity of absolute / relative url
-any_path_re = re.compile('^/?[a-zA-Z0-9_.-]+(/[a-zA-Z0-9_.-]+)*/?$') 
+any_path_re = re.compile('^/?[a-zA-Z0-9_.-]+(/[a-zA-Z0-9_.-]+)*/?$')
+
+# checks validity of relative url
+# matches the following:
+# /test
+# /test/
+# ./test/
+# ../test/
+relative_url_regex = re.compile(r'^[^/<>]+/[^/<>].*$|^/[^/<>]*.*$', re.IGNORECASE)
 
 
 def levelize_path(path):
     """Splits given path to list of paths removing latest level in each step.
-    
+
     >>> path = '/application/item/new'
     >>> levelize_path(path)
     ['/application/item/new', '/application/item', '/application']
     """
-    parts = path.rstrip("/").split("/")
-    paths = []
-    for i in range(len(parts), 0, -1):
-        sub_path = ('/').join(parts[:i])
-        if sub_path:
-            paths.append(sub_path)
-    return paths
+    parts = tuple(filter(None, path.split('/')))
+    return ['/' + '/'.join(parts[:n]) for n in range(len(parts), 0, -1)]
 
 
 def urljoin(*segments):
     """Joins url segments together and appends trailing slash if required.
-    
+
     >>> urljoin('a', 'b', 'c')
     u'a/b/c/'
-    
+
     >>> urljoin('a', '//b//', 'c')
     u'a/b/c/'
-    
+
     >>> urljoin('/a', '/b/', '/c/')
     u'/a/b/c/'
-    
+
     >>> urljoin('/a', '')
     u'/a/'
     """
-    cleaned_segments = map(lambda segment: force_unicode(segment).strip("/"), segments)
-    nonempty_segments = filter(lambda segment: segment > "", cleaned_segments)
-    url = ("/").join(nonempty_segments)
-    
-    if segments[0].startswith("/") and not url.startswith("/"):
-        url = "/" + url
-    
-    if settings.APPEND_SLASH and not url.endswith("/"):
-        url += "/"
-    return url
+    url  = '/' if segments[0].startswith('/') else ''
+    url += '/'.join(filter(None, (force_text(s).strip('/') for s in segments)))
+    return url + '/' if settings.APPEND_SLASH else url
 
 
 def is_media_request(request):
@@ -68,6 +68,15 @@ def is_media_request(request):
     return False
 
 
+def static_with_version(path):
+    """
+    Changes provided path from `path/to/filename.ext` to `path/to/$CMS_VERSION/filename.ext`
+    """
+    path_re = re.compile('(.*)/([^/]*$)')
+
+    return re.sub(path_re, r'\1/%s/\2' % (cms.__version__), path)
+
+
 def add_url_parameters(url, *args, **params):
     """
     adds parameters to an url -> url?p1=v1&p2=v2...
@@ -97,6 +106,5 @@ def admin_reverse(viewname, urlconf=None, args=None, kwargs=None, prefix=None,
         urlconf=urlconf,
         args=args,
         kwargs=kwargs,
-        prefix=prefix,
         current_app=current_app
     )
